'' DAC714 Asynchronous Update Object
'' This object is written to use the DAC714 Cascaded Serial Bus Connection with Asynchronous Update.
'' Date: 4/25/2017
''
''      Output Levels for +/-5 Volt Configuration
''      Decimal   Hex           Voltage
''      32,767    $7FFF         +4.999847
''      1         $0001         +0.000153
''      0         $0             0.000000
''      65,535    $FFFF         -0.000153
''      32,768    $8000         -5.000000
''
''

CON
_clkmode = xtal1 + pll16x
_xinfreq = 5_000_000

  POS_FULLSCALE = $7FFF
  NEG_FULLSCALE = $8000
  
  SPI_CLK_DELAY   = 15
  SPI_CLK_STATE   = 1

  '***** ERROR CODES *****
  ERR_NONE               =   0
  ERR_ARG_OUT_OF_RANGE   = 102         ' = Invalid argument (i.e. argument out of range) 
  ERR_INVALID_OPERATION  = 103         ' = Command is invalid for current state.  

OBJ
  SPI             : "SPI_Spin"
    
VAR
  ' Pin assignments
  long data_pin
  long a0_pin
  long a1_pin
  long clr_pin
  ' Last successfully set dac level
  long dac_level

PUB Init(data, a0, a1, clr)
{{ Call this method to initialize the DAC714 object.
   Parameters are the pin numbers assigned for to the DAC714 communication interface.
}}
  dac_level := 0
  
  ' Initialize pin numbers for the DAC instance
  data_pin := data
  a0_pin := a0
  a1_pin := a1
  clr_pin := clr

  SPI.start(SPI_CLK_DELAY, SPI_CLK_STATE)
  
  SetHigh(data_pin)
  SetHigh(a0_pin)
  SetHigh(a1_pin)
  SetHigh(clr_pin)

PUB Clear
{{  Call this method to clear the DAC registers. }}

  SetLow(clr_pin)
  waitcnt(cnt + clkfreq / 100)     ' wait 1ms
  SetHigh(clr_pin)
  return ERR_NONE
  
PUB SetOutputLevel(count) : ErrCode
{{ Call this method to set the output level of the DAC instance. }}

  if(count > $FFFF)
    return ERR_ARG_OUT_OF_RANGE

  SPI.SHIFTOUT(data_pin, a0_pin, SPI#MSBFIRST, 16, count)
                                                          
  SetLow(a1_pin) ' toggle A1 to latch the data

  waitcnt(cnt + clkfreq / 100)     ' wait 1ms
  SetHigh(a1_pin)

  dac_level := count
  return ERR_NONE

PUB SetOutputMaxPositive : ErrCode
  return SetOutputLevel(POS_FULLSCALE)

PUB SetOutputMaxNegative : ErrCode
  return SetOutputLevel(NEG_FULLSCALE)

PUB GetOutputLevel
  return dac_level

PRI SetHigh(Pin)
    dira[Pin]~~
    outa[Pin]~~
         
PRI SetLow(Pin)
    dira[Pin]~~
    outa[Pin]~
        